<?php
/**
 * User login and tracking class
 * @package login
 */
class session {
	static $username=NULL;
	static $guid=NULL;
	static $userlevel=0;
	static $logged_in=false;
	static $userinfo=array();
	static private $userid=NULL;
	static private $dbconn=NULL;
	/**
	 * Time till user is considered inactive
	 */
	static private $inactivetime=600; //10 minute inactivity timeout
	/**
	 * Constructor for the session class
	 */
	static function init() {
		global $local_connection;
		self::$dbconn=$local_connection;
		self::$logged_in=self::checkauth();
	}
	/**
	 * Checks if the user is logged in
	 * @return bool true if logged in, false if not.
	 */
	static private function checkauth() {
		if(isset($_SESSION['userguid'],$_SESSION['userid'])) {
			if(!self::confirmUserId($_SESSION['userguid'],$_SESSION['userid'])) { //Invalid user
				unset($_SESSION['userguid'],$_SESSION['userid']);
				return false;
			}
			// The user is logged in
			self::$guid=$_SESSION['userguid'];
			self::$userinfo=self::getUserInfo(self::$guid);
			self::$username=self::$userinfo['username'];
			self::$userid=self::$userinfo['userid'];
			self::$userlevel = self::$userinfo['userlevel'];
			self::updateTimestamp(self::$guid);
			return true;
		}
		return false;
	}
	/**
	 * Verifies the userid of the user.
	 * @param string $guid The GUID of the user you want to check
	 * @param string $userid The userid you want to verify
	 * @return bool true is the userid matches the guid, false if not.
	 */
	static private function confirmUserId($guid,$userid) {
		$q=self::$dbconn->query('SELECT * FROM `users` WHERE `GUID`=\''.$guid.'\'');
		if($q->num_rows!=1) return false; //No such user
		$res=$q->fetch_assoc();
		if($userid!=$res['userid']) return false; //The userid doesn't match the guid
		if($res['timestamp'] < (time()-self::$inactivetime)) return false;
		return true;
	}
	/**
	 * Gets userinfo by guid
	 * @param string $guid The user's GUID you want to look up
	 * @return array|NULL Array containing user information (names from table fields)
	 */
	static function getUserInfo($guid) {
		$q=self::$dbconn->query('SELECT * FROM `users` WHERE `GUID`=\''.$guid.'\'');
		if($q->num_rows!=1) return NULL;
		return $q->fetch_assoc();
	}
	/**
	 * Logs a user in.
	 * @param string $username The user's username
	 * @param string $password The password the user entered
	 * @return bool Returns true if login was successful, false if an error occured
	 */
	static function login($username,$password) {
		if(strlen($username = trim($username)) == 0) //No username given
			return false;
		if(!preg_match('%^([0-9a-z])*$%', $username)) //Username not alphanumeric
			return false;
		if(strlen($password)==0) //No password given
			return false;
		$password=md5($password); //Hash before any other functions! This is to prevent password leakage due to errors in debug mode.
		if(!self::userExists($username,1)) //User does not exist
			return false;
		$guid=self::$dbconn->query('SELECT `GUID` FROM `users` WHERE `username`=\''.$username.'\'')->fetch_object()->GUID;
		return self::confirmUserPass($guid,$password);
	}
	/**
	 * Checks whether a user exists or not
	 * @param string $guid The user's GUID or username
	 * @param int $by 0 to check by GUID, 1 to check by username
	 * @return bool Returns true if the user exists, false if not.
	 */
	static function userExists($guid,$by=0) {
		if($by==0)
			$q=self::$dbconn->query('SELECT * FROM `users` WHERE `GUID`=\''.$guid.'\'');
		elseif($by==1)
			$q=self::$dbconn->query('SELECT * FROM `users` WHERE `username`=\''.$guid.'\'');
		else {
			trigger_error('Unexpected value of argument 2',E_USER_WARNING); //We do not support your value.
			return;
		}
		return ($q->num_rows==1);
	}
	/**
	 * Checks the user's password
	 * @param string $guid The user's GUID
	 * @param string $phash The hashed password
	 * @return bool Returns true if the password is correct, false if not.
	 */
	static private function confirmUserPass($guid,$phash) {
		$q=self::$dbconn->query('SELECT * FROM `users` WHERE `GUID`=\''.$guid.'\'');
		if($q->num_rows!=1) return false; //No such user
		if($phash==$q->fetch_object()->password){ //The password matches the guid
			self::updateTimestamp($guid);
			$_SESSION['userguid']=$guid;
			$_SESSION['userid']=self::generateId($guid);
			self::$logged_in=self::checkauth();
			return true;
		}
		return false;		
	}
	/**
	 * Logs the current user out
	 */
	static function logout() {
		$q=self::$dbconn->query('UPDATE `users` SET `userid`=NULL WHERE `GUID`=\''.self::guid.'\'');
		unset($_SESSION['userguid'],$_SESSION['userid']);
		self::$username=NULL;
		self::$guid=NULL;
		self::$userlevel=0;
		self::$logged_in=false;
		self::$userinfo=array();
		self::$userid=NULL;
	}
	/**
	 * Registers a new user
	 * @param string $username The username the registrant wants
	 * @param string $password The password the registrant wants to use
	 * @param string $email The registrant's email-address
	 * @return bool true on success, false on failure.
	 * @uses JSError::add()
	 * @uses JSError::count()
	 */
	static function register($username,$password,$email) {
		global $t;
		// Check fields entered
		if(strlen($username = trim($username)) == 0) //Username
			JSError::add($t->_('Please enter a username'));
		if(strlen($password)==0) //Password
			JSError::add($t->_('Please enter a password'));
		if(strlen($email)==0) //Email
			JSError::add($t->_('Please enter an e-mail address'));
		if(JSError::count()!=0) //Some fields not entered
			return false;
		//Check username
		if(strlen($username) < 5) //Length
			JSError::add($t->_('Username too short (at least 5 letters)'));
		if(strlen($username) > 30) //Length
			JSError::add($t->_('Username too long (not more than 30 letters)'));
		if(!preg_match('/^[0-9a-z]+$/',$username)) //Alphanumeric
			JSError::add($t->_('Username must be alphanumeric'));
		if(self::userExists($username,1)) //Not registered
			JSError::add($t->_('User already exists'));
		
		//Check password
		if(strlen($password) < 4) //Length
			JSError::add($t->_('Password too short (at least 4 characters)'));
		
		//Check email
		if(!preg_match("/^([a-zA-Z0-9])+@([a-zA-Z0-9_-])+(\.[a-zA-Z0-9_-]+)+/",$email)) //Structure
			JSError::add($t->_('Please enter a valid e-mail address'));
		if(JSError::count()!=0)
			return false;
		return self::addUser($username,md5($password),$email);
	}
	/**
	 * Add a new user to the database
	 * @param string $username The username to register
	 * @param string $phash The hashed password to register
	 * @param string $email The email address to link to the username
	 * @return bool true on success, false on failure
	 */
	static private function addUser($username,$phash,$email) {
		return (bool) self::$dbconn->query('INSERT INTO `users` VALUES (UUID(),\''.$username.'\',\''.$phash.'\',NULL,1,\''.$email.'\',UNIX_TIMESTAMP(),0,\'a:1:{i:0;s:2:"en";}\')');
	}
	/**
	 * Update the timestamp of the user
	 * @param string $guid The guid of the user to update the timestamp
	 */
	static private function updateTimestamp($guid) {
		self::$dbconn->query('UPDATE `users` SET `timestamp`=UNIX_TIMESTAMP() WHERE `GUID`=\''.$guid.'\''); //Update the last active timestamp
	}
	/**
	 * Generate a new userid
	 * @param string $guid The guid of the user to generate a new userid for
	 * @return string The generated id
	 */
	static private function generateId($guid) {
		$genid=md5(uniqid(mt_rand(),true));
		self::$dbconn->query('UPDATE `users` SET `userid`=\''.$genid.'\' WHERE `GUID`=\''.$guid.'\'');
		return $genid;
	}
	static function isAdmin() {
		return (self::$userlevel==9);
	}
	/**
	 * Update a field in the user database
	 * @param string $guid The GUID of the user information you want to update
	 * @param string $field The fieldname you want to update
	 * @param string $content The new content for the record
	 * @return bool true on success, false on failure
	 */
	static function updateUserField($guid,$field,$content) {
		return self::$dbconn->query("UPDATE `users` SET `$field` = '$content' WHERE `GUID` = '$guid'");
	}
	static function editPass($oldpass,$newpass) {
		if(self::confirmUserPass(self::$guid,md5($oldpass)))
			return self::updateUserField(self::$guid,'password',md5($newpass));
		return false;
	}
	/**
	 * Gets the user's username from the GUID
	 * @param string $guid The GUID of the user you want to look up
	 * @return string the username
	 */
	static function getUser($guid) {
		if(!self::userExists($guid)) return 'Unknown user';
		return self::$dbconn->query('SELECT `username` FROM `users` WHERE `GUID`=\''.$guid.'\'')->fetch_object()->username;
	}
	/**
	 * Gets the user's GUID from the username
	 * @param string $username The username of the user you want to look up
	 * @return string the GUID
	 */
	static function getGUID($username) {
		if(!self::userExists($username,1)) return 0;
		return self::$dbconn->query('SELECT `GUID` FROM `users` WHERE `username`=\''.$username.'\'')->fetch_object()->GUID;
	}
	/**
	 * Log user actions
	 * @param int $type The type of action to log (200 for success, 401 for authentication failure,...)
	 * @param string $msg The message to log
	 */
	static  function log($type,$msg) {
		self::$dbconn->query("INSERT INTO serv_log VALUES (NULL,$type, NULL, '".self::$guid."', '$_SERVER[REMOTE_ADDR]', '$_SERVER[REQUEST_URI]', '$msg', 0)");
	}
}
session::init();