<?php //@./inc/httpmysql/httpmysql.php ?>
<?php
/*****************************************\
| HTTPMySQL - Query a remote MySQL server |
| through the HTTP protol.                |
\*****************************************/
if(function_exists('curl_init'))  //Load the cURL connector
	require('curl.php');
else //Load socket
	require('fsock.php');
class HTTPMySQL extends HTTPMySQL_connect {
	private $query_url=NULL;
	private $user=NULL;
	private $pass=NULL;
	private $status=false;
	private $version='1.0,1.1,1.3,1.4';
	/**
	 * Open a new connection to the HTTPMySQL script
	 * @param string $host Can be either a host name or an IP address.
	 * @param string $user The HTTPMySQL script username
	 * @param string $pass The HTTPMySQL script password
	 * @param string $url The place where the HTTPMySQL script resides
	 */
	function __construct($host,$user,$pass,$url) {
		$this->host=$host;
		$this->query_url=$url;
		$this->user=$user;
		$this->pass=$pass;
		$headers=unserialize($this->post_request($this->host,$this->query_url,'check=1&user='.$user.'&pass='.$pass));
		if(!in_array($headers['Version'], explode(',',$this->version))) {
			return false;
		}
		if($headers['Status']!==true) {
			$errors=strip_tags(base64_decode($headers['ErrorCode']));
			$this->status=false;
			return false;
		}
		$this->status=true;
	}
	/**
	 * Performs a query on the database
	 * @param string $q The query string. Data inside the query should be properly escaped.
	 * @return HTTPMySQL_result Returns FALSE on failure. For successful SELECT, SHOW, DESCRIBE or EXPLAIN queries mysqli_query() will return a result object. For other successful queries mysqli_query() will return TRUE.
	 */
	function query($q) {
		$query=@unserialize($this->post_request($this->host,$this->query_url,'query='.urlencode($q).'&user='.$this->user.'&pass='.$this->pass));
		if($query===false) return false;
		if($query===true) return true;
		return new HTTPMySQL_result($query);
	}
	function ping() {
		return $this->status;
	}
}
class HTTPMySQLi extends HTTPMySQL { //For people who use the wrong name...
}

class HTTPMySQL_result {
	private $query;
	private $current_row=0;
	/**
	 * Gets the number of rows in a result
	 * @var int
	 */
	public $num_rows=0;
	/**
	 * Returns the lengths of the columns of the current row in the result set
	 * @var array
	 */
	public $lenghts=false;
	/**
	 * Get the number of fields in a result
	 * @var int
	 */
	public $field_count=0;
	function __construct($result) {
		if($result['Status']!==true) {
			$errors=strip_tags(base64_decode($headers['Error']));
			return false;
		}
		$this->query=$result;
		$this->num_rows=count($result['Result']);
		if($this->num_rows>0)
			$this->field_count=count($result['Result'][0]);

	}
	/**
	 * Fetch a result row as an associative array
	 * @return array Returns an associative array that corresponds to the fetched row or NULL if there are no more rows.
	 */
	function fetch_assoc() {
		$q=$this->query['Result'];
		$this->lenghts==false;
		if(count($q)>=$this->current_row) {
			$v=$q[$this->current_row];
			$this->current_row++;
		}
		else {
			$this->lengths=false;
			return NULL;
		}
		$i=1;
		$this->lengths=array();
		foreach ($v as $value) {
			$this->lenghts[]=strlen($value);
		}
		return $v;
	}
	/**
	 * Fetch a result row as an associative, a numeric array, or both
	 * @param int $resulttype This optional parameter is a constant indicating what type of array should be produced from the current row data. The possible values for this parameter are the constants MYSQLI_ASSOC, MYSQLI_NUM, or MYSQLI_BOTH.
	 * @return array Returns an array of strings that corresponds to the fetched row or NULL if there are no more rows in resultset.
	 */
	function fetch_array($resulttype=MYSQLI_BOTH) {
		if($resulttype==MYSQLI_ASSOC)
			return $this->fetch_assoc();
		if($resulttype==MYSQLI_NUM)
			return $this->fetch_row();
		if($resulttype==MYSQLI_BOTH) {
			$a1=$this->fetch_assoc();
			$this->current_row--;
			$a2=$this->fetch_row();
			if($a1==NULL||$a2==NULL) return NULL;
			return array_merge($a1,$a2);
		}
	}
	/**
	 * Get a result row as an enumerated array
	 * @return array returns an array of strings that corresponds to the fetched row or NULL if there are no more rows in result set.
	 */
	function fetch_row() {
		if(($v=$this->fetch_assoc())===NULL) return NULL;
		return array_values($v);
	}
	/**
	 * Returns the current row of a result set as an object
	 * @return stdClass Returns an object with string properties that corresponds to the fetched row or NULL if there are no more rows in resultset.
	 */
	function fetch_object() {
		if(($v=$this->fetch_assoc())===NULL) return NULL;
		$o=new stdClass();
		foreach ($v as $k => $x) {
			$o->$k=$x;
		}
		return $o;
	}
	/**
	 * Frees the memory associated with a result
	 * @internal Trying to do as much as possible, but we can't destroy our class :(
	 */
	function free() {
		unset($this->query,$this->current_row,$this->num_rows,$this->field_count,$this->lenghts);
	}
	/**
	 * @ignore
	 */
	function free_result() {
		$this->free();
	}
	/**
	 * @ignore
	 */
	function close() {
		$this->free();
	}
}